"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const zmq = __importStar(require("zeromq"));
const kernel = require('./ipker.json');
const connectkernel_1 = require("../communication/connectkernel");
const nodeCleanup = require('node-cleanup');
class Executor {
}
Executor.sessions = [];
Executor.latestIoPort = 55548;
Executor.latestShellPort = 51314;
Executor.openSession = (scope) => __awaiter(this, void 0, void 0, function* () {
    let session = Executor.sessions.find((e) => e.id === scope);
    if (!session) {
        session = yield Executor.createSession(scope);
        session.socketSubscriber.connect(kernel.transport + "://" + kernel.ip + ":" + session.ioPort);
        session.socketDealer.connect(kernel.transport + "://" + kernel.ip + ":" + session.shellPort);
        session.socketSubscriber.subscribe("stream");
        Executor.handleSubscriberMsg(session.socketSubscriber);
        Executor.handleDealerMsg(session.socketDealer, session);
    }
});
Executor.exit = (scope) => __awaiter(this, void 0, void 0, function* () {
    let session = Executor.sessions.find((e) => e.id === scope);
    yield session.kernel.cleanup();
});
Executor.createSession = (id) => __awaiter(this, void 0, void 0, function* () {
    const kernel = yield connectkernel_1.startKernel(Executor.latestShellPort, Executor.latestIoPort);
    nodeCleanup(function () {
        kernel.cleanup()
            .then(() => {
            nodeCleanup.uninstall();
        })
            .catch(() => {
            nodeCleanup.uninstall();
        });
        return false;
    });
    const session = {
        id,
        parent_header: {},
        last_header: {
            msg_id: "init_id",
            msg_type: "execute_request",
            username: 'pipcook',
            version: "5.0",
            session: "pipcook_session"
        },
        dealerMsg: {},
        socketDealer: new zmq.Dealer,
        socketSubscriber: new zmq.Subscriber,
        timer: null,
        ioPort: Executor.latestIoPort,
        shellPort: Executor.latestShellPort,
        kernel
    };
    Executor.latestIoPort++;
    Executor.latestShellPort++;
    Executor.sessions.push(session);
    return session;
});
Executor.handleSubscriberMsg = (socketSubscriber) => {
    socketSubscriber.receive()
        .then((msg) => {
        msg.forEach((item, index) => {
            try {
                const json = JSON.parse(item.toString());
                if (json.name === 'stdout') {
                    console.log('[PYTHON: ]', json.text);
                }
                else if (json.name === 'stderr') {
                    console.error('[PYTHON: ]', json.text);
                }
            }
            catch (e) {
            }
        });
        Executor.handleSubscriberMsg(socketSubscriber);
    })
        .catch((err) => {
        Executor.handleSubscriberMsg(socketSubscriber);
    });
};
Executor.handleDealerMsg = (socketDealer, session) => {
    socketDealer.receive()
        .then((msg) => {
        const message = {};
        msg.forEach((item) => {
            try {
                const json = JSON.parse(item.toString());
                if (json.msg_type === 'execute_request') {
                    message.requestId = json.msg_id;
                }
                if (json.msg_type === 'execute_reply') {
                    message.responseId = json.msg_id;
                }
                if (json.status && json.execution_count !== undefined) {
                    if (json.status === 'ok') {
                        message.status = true;
                    }
                    else {
                        message.status = false;
                        message.traceback = json.traceback;
                    }
                }
            }
            catch (err) {
            }
        });
        session.dealerMsg = message;
        Executor.handleDealerMsg(socketDealer, session);
    })
        .catch((err) => {
        console.error(err);
    });
};
Executor.execute = (scope, code) => {
    const session = Executor.sessions.find((e) => e.id === scope);
    if (!session) {
        throw new Error('no session Found!');
    }
    const header = session.last_header;
    const parent_header = session.parent_header;
    session.parent_header = header;
    const metadata = {};
    const content = {
        code: code,
        silent: true,
        user_expressions: {}
    };
    session.socketDealer.send(["", "<IDS|MSG>", "",
        JSON.stringify(header),
        JSON.stringify(parent_header),
        JSON.stringify(metadata),
        JSON.stringify(content)
    ]);
    return new Promise((resolve, reject) => {
        session.timer = setInterval(() => {
            if (session.dealerMsg && session.dealerMsg.requestId === header.msg_id) {
                session.last_header.msg_id = session.dealerMsg.responseId;
                if (session.dealerMsg.status) {
                    clearInterval(session.timer);
                    resolve();
                }
                else {
                    if (Array.isArray(session.dealerMsg.traceback)) {
                        session.dealerMsg.traceback.forEach((item) => {
                            console.error(item);
                        });
                    }
                    else {
                        console.error('[PYTHON: ]', session.dealerMsg.traceback);
                    }
                    clearInterval(session.timer);
                    reject();
                }
            }
        }, 1);
    });
};
exports.default = Executor;
//# sourceMappingURL=communication.js.map